/*:
 * @plugindesc VIPツクスレの情報を取得するよ！
 * @help
 * このプラグインにはプラグインコマンドとかは無いよ！
 * スクリプトから使ってね！
 * 最新版のツクールMVと最新のChromeかFirefoxじゃないと動かないかもしれないから気を付けてね！
 *
 * あと、サーバーに負担をかけないように、
 * 再通信はなるべく早くとも10秒に1回ぐらいのペースにしようね！
 *
 * このプラグインでは、VIPRPG臨時アップローダー兼簡易自動保管庫に書かれていた記事を元に、
 * 2ch.scからデータをスクレイピングすることで動作します。
 *
 * vipGetTksr() // ツクスレ取得を開始する
 * vipTksrIsLoading // ツクスレの取得処理が行われている間はtrueになる
 * vipTksrIsError // ツクスレの取得中にエラーが発生し中断したときにtrueになる
 * vipCacheTksr // ツクスレの取得が完了したときに全レスのデータが入った配列が入る
 *
 * バグあったらゴメス。
 * JavaScript強い人は好きに改造してクレメンス
 *
 * vipGetTksrの処理中は、次のような処理が動きます
 * 1: subject.txtからRPGツクールとタイトルに入ってるスレを探す
 * 2: 該当するスレの>>1にtkool.jpが入ってるか調べる
 * 3: tkool.jpが入ってるスレの投稿を集計する
 * なので似たようなスレが複数立ってると、そのぶん時間がかかります
 * >>1にtkool.jpがあるスレが2つ以上ある場合はsubject.txtで見たときの順番が一番上にあるスレを使います。
 *
 * ===== 使い方 =====
 *
 * ◆注釈：ツクスレ取得を開始する
 * ◆スクリプト：vipGetTksr();
 *
 * ◆注釈：ツクスレが取得されるまで待つ（3～5秒ぐらいかかる）
 * ◆ループ
 * 　◆注釈：vipTksrIsLoadingがtrue状態の間は読みこみ中
 * 　◆条件分岐：スクリプト：vipTksrIsLoading === false
 * 　　◆ループの中断
 * 　　◆
 * 　：分岐終了
 * 　◆ウェイト：1フレーム
 * 　◆
 * ：以上繰り返し
 *
 * ◆注釈：vipCacheTksrにツクスレのデータが入ってるのでお好きなように！
 * ◆スクリプト：
 * // >>50を取得する
 * var res = vipCacheTksr[50];
 *
 * // 名前を変数1に代入する
 * $gameVariables.setValue(1, res.name);
 * // メールアドレスを変数2に代入する
 * $gameVariables.setValue(2, res.email);
 * // sageかどうかをスイッチ3に代入する
 * $gameSwitches.setValue(3, res.sage);
 * // 日付文字列を変数4に代入する
 * $gameVariables.setValue(4, res.date);
 * // 時間文字列を変数5に代入する
 * $gameVariables.setValue(5, res.time);
 * // ID文字列を変数6に代入する
 * $gameVariables.setValue(6, res.id);
 * // 本文を変数7に代入する
 * $gameVariables.setValue(7, res.body);
 *
 * ◆注釈：上の例は以下と同じ！
 * ◆変数の操作：#0001 = vipCacheTksr[50].name
 * ◆変数の操作：#0002 = vipCacheTksr[50].email
 * ◆条件分岐：スクリプト：vipCacheTksr[50].name === true
 * 　◆スイッチの操作：#0003 = ON
 * ：それ以外のとき
 * 　◆スイッチの操作：#0003 = OFF
 * ：分岐終了
 * ◆変数の操作：#0004 = vipCacheTksr[50].date
 * ◆変数の操作：#0005 = vipCacheTksr[50].time
 * ◆変数の操作：#0006 = vipCacheTksr[50].id
 * ◆変数の操作：#0007 = vipCacheTksr[50].body
 *
 * ◆注釈：変数7に入れたレス本文を表示！
 * ◆文章：なし, ウィンドウ, 下
 * ：　　：\v[7]
 *
 * ===== 上級者向けの使い方 =====
 *
 * JavaScript得意な人のためのテクニック
 *
 * // スレのIDを全部集計して文字列配列を返す
 * var tktk = vipCacheTksr.reduce((a, b) => [...a, b.id], [])
 *
 * // 保守が含まれるレスだけを抽出
 * var tktk = vipCacheTksr.filter(a => a.body.match('保守'))
 *
 * ===== 改造する人向け =====
 *
 * 2ch.scからのデータは文字コードに難があるのでTextDecoderで変換してます。
 * あと、なぜか処理中の文字列をencodeしないと2バイト以上の文字がmatchに反応しない謎のバグがあったのでいちいちencode/decodeしてます。（ちょっとスマートじゃない）
 *
 */
;(function() {
    // スレッド一覧
    const LIST_URL = 'https://viper.2ch.sc/news4vip/subject.txt'
    // DAT置き場
    const GET_DAT_URL = (a) => 'https://viper.2ch.sc/news4vip/dat/' + a
    // RPGツクールをencodeしたもの
    const VIPRPG_THREAD_ALIAS = 'RPG%E3%83%84%E3%82%AF%E3%83%BC%E3%83%AB'
    // これが>>1にあったらツクスレ判定
    const DECISION_WORD = 'tkool.jp'
    // エラーコード
    const NOT_FOUND = 'スレッドが見つかりませんでした'
    const THRED_IS_MULTI =
        'スレッドが2つ以上見つかりました。ダメもとで1つ目の候補を使用します。'
    // 2ch.scからのデータを文字化けせずに持ってくる
    const scfetch = async (a) =>
        new TextDecoder('Shift_JIS').decode(
            await (await fetch(a)).arrayBuffer()
        )
    // ウェイト
    const wait = async (a) => new Promise((b) => setTimeout(b, a * 1000))
    // <>区切りの文字列を区切る
    const scsplit = (a) =>
        a.replace(/(^.*)?<>(.*?)\s\((.*?)\)$/, '$1<>$2<>$3').split('<>')
    // ↑かつencodeする
    const splitEncode = (a) => scsplit(a).map(encodeURIComponent)
    // レスをXMLにして文字列化することで不要なHTMLタグをなくす
    const getText = (a) =>
        new DOMParser()
            .parseFromString(`<res>${a}</res>`, 'text/xml')
            .querySelector('res').textContent

    // 初期値
    window.vipTksrIsLoading = false
    window.vipTksrIsError = false

    // メイン処理
    const main = async () => {
        // 準備
        window.vipTksrIsLoading = true
        window.vipTksrIsError = false
        // 怒涛のメソッドチェーンがメイン処理を襲う！！！！！
        const result = ( // スレ候補が複数見つかったら順次分析
            await Promise.all([
                await wait(1), // ウェイトおいしいです
                // スレ一覧からめぼしいスレタイとdatを探す
                ...(await scfetch(LIST_URL))
                    .split('\n')
                    .map(splitEncode)
                    .filter((a) => a.length === 3)
                    .filter((a) => a[1].match(VIPRPG_THREAD_ALIAS))
                    .map((a) => a.map(decodeURIComponent))
                    .map(([a]) => a)
                    .map(async (a, b) => {
                        await wait(b)
                        return (await scfetch(GET_DAT_URL(a)))
                            .split('\n')
                            .map(splitEncode)
                    })
            ])
        )
            .slice(1) // それはしょうゆです
            // >>1にめぼしいワードがあればそれはツクスレだ！！！
            .filter((a) => a[1][3].match(DECISION_WORD))
        // ない
        if (result.length === 0) throw new Error(NOT_FOUND)
        // なんかたくさんあるんだけど
        if (result.length !== 1) console.warn(THRED_IS_MULTI)
        // ここでスレの情報を分析して使いやすく料理する
        const final = result[0]
            .filter((a) => a.length === 5)
            .map((a, num) => {
                const [name, email, data, body, meta] = a
                const [date, time, id] = decodeURIComponent(data).split(/\s?\s/)
                const mail = decodeURIComponent(email)
                return {
                    num,
                    name: decodeURIComponent(name),
                    email: mail,
                    sage: mail === 'sage',
                    data: decodeURIComponent(data),
                    date,
                    time,
                    id,
                    body: getText(
                        decodeURIComponent(
                            body
                                .replace(/%3Cbr%3E/g, '\n')
                                .replace(/%26amp%3B/g, '&')
                                .replace(/%26lt%3B/g, '<')
                                .replace(/%26gt%3B/g, '>')
                                .replace(/%26%2339%3B/g, "'")
                                .replace(/%26quot%3B/g, '"')
                        )
                    ),
                    meta: decodeURIComponent(meta)
                }
            })
        // めでてえｗｗｗｗｗｗｗ
        window.vipTksrIsLoading = false
        window.vipCacheTksr = final
        return final
    }

    window.vipGetTksr = () => {
        main()
            .then(console.log)
            .catch(() => {
                // めでたくねえ・・・
                window.vipTksrIsError = true
            })
    }
})()
